# locpolreg_no_plot.R Non-parametric estimation for the r-th derivative
#             of the regression function m(x) by local polinomial fitting.
#
# Input: 
#      x,y  Pairs of observations (n,1)
#      h    Smoothing parameter
#      p    degree for the polinomial locally fitted (default=1)
#      r    derivative of m(x) to be estimated (default=0)
#      xg   points where the r-th derivative of m(x) is estimated (default=x)
#
# Output:
#      mxgr Estimation of the r-th derivative of m(x) at xg
#
# Uso:
#      result <- locpolreg_no_plot(x,y,h,p,r,xg)
# 'result' is a list with two elements: 
#      'mxgr' the required estimation
#      'S'    the smoothing matriz giving this estimation
# 
locpolreg_no_plot <- function(x,y,h=(max(x)-min(x))/5,p=1,r=0,xg=x,...){
   if (sum(diff(xg)<0)>0) xg <- sort(xg)
      
   n <- length(x);
   m <- length(xg);
   mxgr <- seq(1,m)*0;
   S <- matrix(0,nrow=m,ncol=n)

   factr <- max(1,prod(1:r))

   for (i in seq(1,m)){
      Ih <- abs(x-xg[i])<h;
      n <- sum(Ih);     
      xh <- x[Ih]-xg[i];
      Dp <- matrix(1,nrow=n,ncol=p+1);
      if (p>0){for (j in 1:p) Dp[,j+1] <- xh^j}
      Wx <- epan(xh/h)/h;
      Wm <- Wx%*%ones(1,p+1);
      Dpp <- Wm*Dp;
      Si <- solve(t(Dp)%*%Dpp)%*%t(Dpp);
      beta <- Si%*%y[Ih];
      mxgr[i] <- factr*beta[r+1];
      S[i,Ih] <- Si[r+1,]
   }
  
return(list(mxgr=mxgr,S=S))
}

epan <- function(x){.75*(x+1)*(1-x)}
ones <- function(n,m){matrix(1,nrow=n,ncol=m)}
